/**
 * WordPress dependencies
 */
import { combineReducers } from '@wordpress/data';

/**
 * Internal to edit-navigation package.
 *
 * Stores menuItemId -> clientId mapping which is necessary for saving the navigation.
 *
 * @param {Object} state  Redux state
 * @param {Object} action Redux action
 * @return {Object} Updated state
 */
export function mapping( state, action ) {
	const { type, postId, ...rest } = action;
	if ( type === 'SET_MENU_ITEM_TO_CLIENT_ID_MAPPING' ) {
		return { ...state, [ postId ]: rest.mapping };
	}

	return state || {};
}

/**
 * Internal to edit-navigation package.
 *
 * Enables serializeProcessing action wrapper by storing the underlying execution
 * state and any pending actions.
 *
 * @param {Object} state  Redux state
 * @param {Object} action Redux action
 * @return {Object} Updated state
 */
export function processingQueue( state, action ) {
	const { type, postId, ...rest } = action;
	switch ( type ) {
		case 'START_PROCESSING_POST':
			return {
				...state,
				[ postId ]: {
					...state[ postId ],
					inProgress: true,
				},
			};

		case 'FINISH_PROCESSING_POST':
			return {
				...state,
				[ postId ]: {
					...state[ postId ],
					inProgress: false,
				},
			};

		case 'POP_PENDING_ACTION':
			const postState = { ...state[ postId ] };
			if ( 'pendingActions' in postState ) {
				postState.pendingActions = postState.pendingActions?.filter(
					( item ) => item !== rest.action
				);
			}
			return {
				...state,
				[ postId ]: postState,
			};

		case 'ENQUEUE_AFTER_PROCESSING':
			const pendingActions = state[ postId ]?.pendingActions || [];
			if ( ! pendingActions.includes( rest.action ) ) {
				return {
					...state,
					[ postId ]: {
						...state[ postId ],
						pendingActions: [ ...pendingActions, rest.action ],
					},
				};
			}
			break;
	}

	return state || {};
}

/**
 * Reducer keeping track of selected menu ID.
 *
 * @param {number} state  Current state.
 * @param {Object} action Dispatched action.
 * @return {Object} Updated state.
 */
export function selectedMenuId( state = null, action ) {
	switch ( action.type ) {
		case 'SET_SELECTED_MENU_ID':
			return action.menuId;
	}

	return state;
}

/**
 * Reducer tracking whether the inserter is open.
 *
 * @param {boolean|Object} state        Current state.
 * @param {Object}         action       Dispatched action.
 * @param {string}         action.type  String indicating action type.
 * @param {boolean}        action.value Flag indicating whether the panel should be open/close.
 */
function blockInserterPanel( state = false, action ) {
	switch ( action.type ) {
		case 'SET_IS_INSERTER_OPENED':
			return action.value;
	}
	return state;
}

export default combineReducers( {
	mapping,
	processingQueue,
	selectedMenuId,
	blockInserterPanel,
} );
